// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import GMAppData

struct ActivitiesTimeline<FooterView: View>: View {
    let entryPrefix: String
    let activities: [Activity]
    let dueDate: Date?
    let labelTitle: String
    let labelColor: Color
    let isFirstInTimeline: Bool
    let backgroundColor: Color
    let lineColor: Color
    let footerView: FooterView

    private let topSpace: CGFloat = 20
    private let cardSpacing: CGFloat = 8

    private var indexForDueDate: Int? {
        guard let dueDate = dueDate else {
            return nil
        }
        return activities.firstIndex(where: { $0.sortDate?.isAfterOrEqual(dueDate) == true })
    }

    var body: some View {
        VStack(spacing: 0) {
            CompatibleVStack(spacing: 0) {
                Group {
                    if isFirstInTimeline {
                        Rectangle()
                            .fill(Color.clear)
                            .frame(height: topSpace) //Space
                    } else {
                        HStack {
                            VerticalLineView(color: lineColor)
                                .frame(height: topSpace)
                            Spacer()
                        }
                        .padding(.leading)
                    }
                }
                .id("\(entryPrefix)_top")

                if !activities.isEmpty {
                    ActivitiesTimelineLabel(
                        title: labelTitle,
                        color: labelColor
                    )
                    .padding(.leading, 8)
                    .id("\(entryPrefix)_beginning_line")
                }
                // Copying list to local variable, since we have seen some weird indexing issue on this.
                // (earlier seen as crashes, but that approach has been changed now)
                let forEachActivities = activities.map { $0 }
                ForEach(forEachActivities.indices) { index in
                    let topPadding: CGFloat = index == 0 ? topSpace : cardSpacing

                    // Inset due date card before card for chosen index
                    if let dueDate: Date = dueDate, index == indexForDueDate {
                        dueDateCard(dueDate: dueDate, topPadding: topPadding)
                    }

                    if let activity: Activity = forEachActivities[safe: index] {
                        ActivityTimelineEntryView(
                            lineColor: lineColor,
                            dotColor: .colorBySeed(activity.careplanId),
                            topPadding: topPadding,
                            content: {
                                ActivityEntryCardView(
                                    activity: activity
                                )
                            }
                        )
                        .id("\(entryPrefix)_\(activity.id ?? index.description)")
                    }
                }
                // Inset due date card as the last object in the list.
                if let dueDate: Date = dueDate, indexForDueDate == nil {
                    dueDateCard(dueDate: dueDate, topPadding: forEachActivities.isEmpty ? topSpace : cardSpacing)
                }
            }
            footerView
                .id("\(entryPrefix)_footer")
        }
        .background(backgroundColor)
    }

    @ViewBuilder
    private func dueDateCard(dueDate: Date, topPadding: CGFloat) -> some View {
        ActivityTimelineEntryView(
            lineColor: lineColor,
            dotColor: R.color.text.color(),
            topPadding: topPadding,
            content: {
                DueDateCardView(dueDate: dueDate)
            }
        )
        .id("\(entryPrefix)_duedate")
    }

    static func isDueDateBetween(_ dueDate: Date, activity1: Activity?, activity2: Activity?) -> Bool {
        dueDate.isBetween(date1: activity1?.sortDate, date2: activity2?.sortDate)
    }
}

#if DEBUG
struct ActivitiesTimeline_Previews: PreviewProvider {
    static var previews: some View {
        ScrollView {
            VStack(spacing: 0) {
                ActivitiesTimeline(
                    entryPrefix: "1",
                    activities: [loadPreviewCarePlan().activities!.first!],
                    dueDate: nil,
                    labelTitle: "Title 1",
                    labelColor: .blue,
                    isFirstInTimeline: true,
                    backgroundColor: .gray,
                    lineColor: .pink,
                    footerView: HStack {
                        VerticalLineView(color: .pink)
                            .frame(height: 30)
                        Spacer()
                    }.padding(.leading)
                )
                ActivitiesTimeline(
                    entryPrefix: "2",
                    activities: loadPreviewCarePlan().activities!,
                    dueDate: nil,
                    labelTitle: "Title 2",
                    labelColor: .blue,
                    isFirstInTimeline: false,
                    backgroundColor: .white,
                    lineColor: .pink,
                    footerView: Rectangle().fill(Color.pink).frame(size: 50)
                )
            }
        }
    }
}
#endif
