// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import PDFServices
import ErrorData
import UserSettingsServices

extension DocumentationView {
    final class ViewModel: ObservableObject {
        let title: String = Localizable.more_documentation_title()
        @Published var isPersonalDocumentLoading: Bool = false
        @Published var isEmployerDocumentLoading: Bool = false
        @Published var documentWasSent: Bool = false

        @Published var error: Error?

        @DI private var pdfService: PDFService
        @DI private var maternityEnvironmentStore: MaternityDataEnvironmentStore
        @DI private var userSettings: UserSettingsService

        private var cancelBag = CancelBag()
        private let fifteenMinutes: TimeInterval = 15 * 60

        var canSendEmployerDocument: Bool {
            canSendDocuments && documentationTimeExpired(forLastSentKey: .lastEmployerDocumentationSentDate)
        }

        var canSendPersonalDocument: Bool {
            canSendDocuments && documentationTimeExpired(forLastSentKey: .lastPersonalDocumentationSentDate)
        }

        private var canSendDocuments: Bool {
            maternityEnvironmentStore.selectedMaternity != nil
        }

        private func documentationTimeExpired(forLastSentKey key: UserSettingsKey<[String: Date]>) -> Bool {
            guard let maternityId = maternityEnvironmentStore.selectedMaternity?.id,
                  let lastDate = userSettings.get(key: key)?[maternityId] else {
                return true // Act as expired if it never was set
            }
            return lastDate.timeIntervalSinceNow <= -fifteenMinutes
        }

        func sendPersonalDocument() {
            guard let maternityId: String = maternityEnvironmentStore.selectedMaternity?.id, canSendPersonalDocument else {
                error = ServiceError.unexpectedError(nil)
                return
            }
            sendPDF(
                publisher: pdfService.sendPersonalPDF(maternityId: maternityId),
                maternityId: maternityId,
                isLoading: { [weak self] in self?.isPersonalDocumentLoading = $0 },
                userSettingsKey: .lastPersonalDocumentationSentDate
            )
        }

        func sendEmployerDocument() {
            guard let maternityId: String = maternityEnvironmentStore.selectedMaternity?.id, canSendEmployerDocument else {
                error = ServiceError.unexpectedError(nil)
                return
            }
            sendPDF(
                publisher: pdfService.sendEmployerPDF(maternityId: maternityId),
                maternityId: maternityId,
                isLoading: { [weak self] in self?.isEmployerDocumentLoading = $0 },
                userSettingsKey: .lastEmployerDocumentationSentDate
            )
        }

        private func sendPDF(publisher: AnyPublisher<Void, ServiceError>, maternityId: String, isLoading: @escaping (Bool) -> Void, userSettingsKey: UserSettingsKey<[String: Date]>) {
            isLoading(true)
            publisher
                .receiveOnMain()
                .sink(
                    receiveCompletion: { [weak self] completion in
                        switch completion {
                        case .failure(let error):
                            self?.error = error
                        case .finished:
                            self?.documentWasSent = true
                            var datesDictionary = self?.userSettings.get(key: userSettingsKey) ?? [:]
                            datesDictionary[maternityId] = Date()
                            self?.userSettings.save(datesDictionary, for: userSettingsKey)
                        }
                        isLoading(false)
                    },
                    receiveValue: { _ in }
                )
                .store(in: &cancelBag)
        }
    }
}
