// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import MaternityServices
import GMAppData
import CMSServices
import CMSData
import ErrorData
import Combine

extension OverviewView {
    class ViewModel: ObservableObject {
        private var cancelBag = CancelBag()
        @DI private var maternityEnvironmentStore: MaternityDataEnvironmentStore
        @DI private var articlesService: ArticlesService

        // Kept in this view model to avoid creating new instances on EnvironmentObject changes...
        private (set) var pregnancyPickerViewModel = OverviewPregnancyPickerView.ViewModel()

        @Published var state: DataLoadingState = .initial
        @Published var presentDidGiveBirthModal: Bool = false
        @Published var presentSendFeedback: Bool = false
        @Published var presentMaternityPicker: Bool = false
        @Published var pushKnowledgeSection: Bool = false
        @Published var shouldReloadData: Bool = false

        @Published var currentRelevantArticleResults: [CMSData.Category] = []
        @Published var nextRelevantArticleResults: [CMSData.Category] = []

        var navigationBarTitle: String {
            maternityEnvironmentStore.fullUserName ?? ""
        }

        private var currentWeekForArticles: Int {
            guard let week = maternityEnvironmentStore.selectedMaternity?.week else {
                return 1
            }
            return week
        }

        //Has to be kept here, to avoid recreation of view model on environment changes, while sheet is present.
        let dateOfBirthViewModel = OverviewDateOfBirthView.ViewModel()

        var pregnancyState: OverviewPregnancyState {
            if maternityEnvironmentStore.selectedMaternity == nil {
                return .none
            } else if maternityEnvironmentStore.selectedMaternity?.isEnded == true {
                return .ended
            } else if hasDateOfBirth {
                return .activeAndGaveBirth
            } else {
                return .activeAndWaitingForBirth
            }
        }

        var hasDateOfBirth: Bool {
            maternityEnvironmentStore.dateOfBirthForSelectedMaternity != nil
        }

        var endedDate: Date? {
            maternityEnvironmentStore.selectedMaternity?.closeDateTime
        }

        var numberOfFetuses: Int {
            maternityEnvironmentStore.selectedMaternity?.numberOfFetuses ?? 0
        }

        init() {
            listenForReloadEvents()
        }

        private func listenForReloadEvents() {
            // Reload article data when saving on dateOfBirth view
            dateOfBirthViewModel.$didSave
                .sink { [weak self] reload in
                    if reload {
                        self?.fetchData(force: true)
                    }
                }
                .store(in: &cancelBag)
        }

        func fetchData(force: Bool = false) {
            // We don't want to fetch articles on empty state.
            // The article-week does not change while logged in and if it's empty it will stay empty until next week (which would cause a new fetch upon new login).
            guard (state.allowsDataFetch && !state.isEmpty) || force else {
                return
            }
            state = .loading

            let articleRequest: Publishers.Zip<AnyPublisher<[CMSData.Category], ServiceError>, AnyPublisher<[CMSData.Category], ServiceError>>
            if hasDateOfBirth {
                let dateOfBirthWeekOffset = maternityEnvironmentStore.dateOfBirthForSelectedMaternity?.weekNumberAfterBirthDate() ?? 1
                articleRequest = articlesService.fetchAfterBirthArticleCategories(for: dateOfBirthWeekOffset)
                    .zip(articlesService.fetchAfterBirthArticleCategories(for: dateOfBirthWeekOffset + 1))
            } else {
                articleRequest = articlesService.fetchArticleCategories(for: currentWeekForArticles)
                    .zip(articlesService.fetchArticleCategories(for: currentWeekForArticles + 1))
            }

            articleRequest
                .receiveOnMain()
                .sink(
                    receiveCompletion: { [weak self] completion in
                        switch completion {
                        case .failure(let error):
                            self?.state = .error(error)
                        case .finished:
                            // Always handle state as data, since knowledge content will handle empty state.
                            self?.state = .data
                        }
                    },
                    receiveValue: { [weak self] (currentCategories, nextCategories) in
                        self?.currentRelevantArticleResults = currentCategories
                        self?.nextRelevantArticleResults = nextCategories
                    }
                )
                .store(in: &cancelBag)
        }
    }

    final class PreviewViewModel: ViewModel {
        override func fetchData(force: Bool = false) {
            currentRelevantArticleResults = Array(CMSData.Category.mockList().prefix(2))
            nextRelevantArticleResults = [ CMSData.Category.mockList().last! ]
            state = .data
        }
    }
}
