// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import Foundation
import SwiftUI
import ThePerfectApp
import UserSettingsServices

/// TabBarController is not using DI for EnvironmentStores, since it creates the views, which has to dependent on the real types and not the protocols ...
final class TabBarController: UITabBarController {
    @DI private var userContext: UserContextProtocol
    @DI private var logger: Logger
    @DI private var maternityEnvironmentStore: MaternityDataEnvironmentStore
    @DI private var userSettings: UserSettingsService

    private var cancelBag = CancelBag()
    private var popper: TabBarControllerPopper?

    // swiftlint:disable:next function_body_length
    override func viewDidLoad() {
        super.viewDidLoad()
        self.popper = TabBarControllerPopper(self) // Assign to strong property

        self.delegate = self.popper // Assign to weak delegate

        let overviewView = OverviewView()
            .environmentObject(MaternityDataEnvironmentStoreImpl.shared)
            .environmentObject(KnowledgeEnvironmentStoreImpl.shared)

        let overviewViewControler = UIHostingController(rootView: overviewView)
        overviewViewControler.tabBarItem = tabbarItem(
            title: Localizable.bottom_navigation_title_overview(),
            image: R.image.icons.tabbar.overview(),
            accessibilityId: "tabbar_overview"
        )

        let knowledgeView = KnowledgeView()
            .environmentObject(KnowledgeEnvironmentStoreImpl.shared)
            .environmentObject(MaternityDataEnvironmentStoreImpl.shared)

        let knowledgeViewController = UIHostingController(rootView: knowledgeView)
        knowledgeViewController.tabBarItem = tabbarItem(
             title: Localizable.bottom_navigation_title_knowledge(),
            image: R.image.icons.tabbar.knowledge(),
            accessibilityId: "tabbar_knowledge"
        )

        let appointmentsView = AppointmentsView()
            .environmentObject(MaternityDataEnvironmentStoreImpl.shared)

        let appointmentViewController = UIHostingController(rootView: appointmentsView)
        appointmentViewController.tabBarItem = tabbarItem(
            title: Localizable.bottom_navigation_title_appointments(),
            image: R.image.icons.tabbar.appointments(),
            accessibilityId: "tabbar_appointments"
        )

        let journalView = JournalView()
            .environmentObject(MaternityDataEnvironmentStoreImpl.shared)
        let journalViewController = UIHostingController(rootView: journalView)
        journalViewController.tabBarItem = tabbarItem(
            title: Localizable.bottom_navigation_title_journal(),
            image: R.image.icons.tabbar.journal(),
            accessibilityId: "tabbar_journal"
        )

        let moreView = MoreView()
            .environmentObject(MaternityDataEnvironmentStoreImpl.shared)
        let moreViewController = UIHostingController(rootView: moreView)
        moreViewController.tabBarItem = tabbarItem(
            title: Localizable.bottom_navigation_title_more(),
            image: R.image.icons.tabbar.more(),
            accessibilityId: "tabbar_more"
        )
        
        if userContext.isAuthenticated {
            viewControllers = [
                overviewViewControler,
                appointmentViewController,
                knowledgeViewController,
                journalViewController,
                moreViewController
            ]
        } else {
            viewControllers = [
                knowledgeViewController
            ]
            tabBar.isHidden = true
        }
        viewControllers?.forEach({ (vc) in
            // Set `UIHostingViewController` backgrounds to navigation bar color to color the status bar area.
            vc.view.backgroundColor = R.color.navigationBar()
            vc.extendedLayoutIncludesOpaqueBars = true
        })
        tabBar.accessibilityIdentifier = "tab_bar"
        tabBar.tintColor = R.color.tabbarSelectedTint()
        tabBar.unselectedItemTintColor = R.color.tabbarDeselectedTint()
        tabBar.barTintColor = R.color.tabbarBackground()

        listenForNotifications()

        if !App.isTesting {
            // Wait half a second to let the user land in the app and then present alerts if needed.
            DispatchQueue.main.asyncAfter(deadline: .now() + .milliseconds(500)) { [weak self] in
                self?.showLoginAlertsIfNeeded()
            }
        }
    }

    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)

        // Check for updates on appearance of tab bar
        if App.buildConfiguration.isTPADistribution {
            logger.info("Checking for TPA update...")
            TPA.shared().checkForUpdate()
        }
    }

    private func tabbarItem(title: String, image: UIImage?, accessibilityId: String) -> UITabBarItem {
        let item = UITabBarItem(
            title: title,
            image: image,
            selectedImage: image
        )
        item.accessibilityIdentifier = accessibilityId
        item.badgeColor = R.color.primary()
        item.setBadgeTextAttributes(
            [
                NSAttributedString.Key.font: FontStyle.tabBarBadge.scaledUIFont() as Any,
                NSAttributedString.Key.foregroundColor: UIColor.white as Any
            ],
            for: .normal
        )
        return item
    }

    private func listenForNotifications() {
        NotificationCenter.default
            .listen(for: .TabBarController.changeTab)
            .sink(receiveValue: handleChangeTabNotification)
            .store(in: &cancelBag)

        NotificationCenter.default
            .listen(for: .TabBarController.changeOfSelectedMaternity)
            .sink(receiveValue: handleChangeOfSelectedMaternityNotification)
            .store(in: &cancelBag)
    }

    private func handleChangeTabNotification(_ notificationValue: ChangeTabNotificationMessage) {
        switch notificationValue {
        case .knowledge:
            selectedIndex = 2
        }
    }

    private func handleChangeOfSelectedMaternityNotification(_ notificationValue: String) {
        // Reset tabs for maternity change
        viewControllers?
            .flatMap({ $0.children.compactMap({ $0 as? UINavigationController }) })
            .forEach({ navCon in
                navCon.popToRootViewController(animated: false)
            })
    }

    func showLoginAlertsIfNeeded() {
        guard userContext.isAuthenticated else {
            return
        }

        let hasMaternities: Bool = (maternityEnvironmentStore.maternities ?? []).isEmpty == false
        let hasActiveMaternity: Bool = (maternityEnvironmentStore.maternities ?? []).contains(where: { $0.isActive })

        if !hasMaternities && userSettings.get(key: .hasSeenNoMaternityCoursesAlert) != true {
            userSettings.save(true, for: .hasSeenNoMaternityCoursesAlert)
            presentOkayAlert(
                title: Localizable.common_no_course_available_alert_title(),
                message: Localizable.common_no_course_available_alert_text()
            )
        } else if !hasActiveMaternity && userSettings.get(key: .hasSeenNoActiveMaternityCoursesAlert) != true {
            userSettings.save(true, for: .hasSeenNoActiveMaternityCoursesAlert)
            presentOkayAlert(
                title: Localizable.common_no_active_course_available_alert_title(),
                message: Localizable.common_no_active_course_available_alert_text()
            )
        } else if hasActiveMaternity {
            // Reset the no-active alert state, when the user actually have an active course.
            userSettings.save(false, for: .hasSeenNoActiveMaternityCoursesAlert)
        }

        if hasMaternities {
            // Reset when there are maternities (they expire after 2 year, which is when we want to show this again)
            userSettings.save(false, for: .hasSeenNoMaternityCoursesAlert)
        }
    }

    private func presentOkayAlert(title: String, message: String) {
        let alert = UIAlertController(
            title: title,
            message: message,
            preferredStyle: .alert
        )
        alert.addAction(.init(title: Localizable.common_okay(), style: .default, handler: nil))
        present(alert, animated: true, completion: nil)
    }
}

extension TabBarController {
    enum ChangeTabNotificationMessage : NotificationCenterValue {
        case knowledge
    }
}

extension String: NotificationCenterValue { }

private class TabBarControllerPopper: NSObject, UITabBarControllerDelegate {
    private var previouslySelectedIndex: Int

    init(_ tabBarController: TabBarController) {
        self.previouslySelectedIndex = tabBarController.selectedIndex
    }

    func tabBarController(_ tabBarController: UITabBarController, didSelect viewController: UIViewController) {
        if previouslySelectedIndex == tabBarController.selectedIndex {
            popToRootOrScrollUp(on: viewController)
        }

        previouslySelectedIndex = tabBarController.selectedIndex
    }

    private func popToRootOrScrollUp(on viewController: UIViewController) {
        let nvc = navigationController(for: viewController)
        let popped = nvc?.popToRootViewController(animated: true)

        if (popped ?? []).isEmpty {
            let rootViewController = nvc?.viewControllers.first ?? viewController
            if let scrollView = firstScrollView(in: rootViewController.view ?? UIView()) {
                let preservedX = scrollView.contentOffset.x
                let y = -scrollView.adjustedContentInset.top
                scrollView.setContentOffset(CGPoint(x: preservedX, y: y), animated: true)
            }
        }
    }

    private func navigationController(for viewController: UIViewController) -> UINavigationController? {
        for child in viewController.children {
            if let nvc = viewController as? UINavigationController {
                return nvc
            } else if let nvc = navigationController(for: child) {
                return nvc
            }
        }
        return nil
    }

    func firstScrollView(in view: UIView) -> UIScrollView? {
        for subview in view.subviews {
            if let scrollView = view as? UIScrollView {
                return scrollView
            } else if let scrollView = firstScrollView(in: subview) {
                return scrollView
            }
        }

        return nil
    }
}
