// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
@testable import TIM
@testable import TIMEncryptedStorage
import Combine
import XCTest

let mockAccessToken = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiJ1c2VySWQiLCJuYW1lIjoiSm9obiBEb2UiLCJpYXQiOjE1MTYyMzkwMjJ9.RT3_bUnfDHAnDNF_Hf1a8OMz7bPSCKr6MD5vyVNdVBA"
let mockRefreshToken = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiJ1c2VySWQiLCJuYW1lIjoiSm9obiBEb2UgcmVmcmVzaCIsImlhdCI6MTUxNjIzOTAyMn0.29ibTzG_6nPhFeDhlOtqOpzA6ERlzGHtFQU-H986gMc"
let mockUserId = "userId"
let mockPassword = "1234"

private struct StoredMockRefreshToken {
    let password: String
    let refreshToken: JWT
}

final class TestMockTIMDataStorage: TIMDataStorage {

    var availableUserIds: Set<String> = []
    private var hasBioAccess: Bool = true
    private var refreshTokensForUserIds: [String: StoredMockRefreshToken] = [:]

    init(hasBioAccess: Bool) {
        self.hasBioAccess = hasBioAccess
    }

    func hasRefreshToken(userId: String) -> Bool {
        refreshTokensForUserIds[userId] != nil
    }

    func hasBiometricAccessForRefreshToken(userId: String) -> Bool {
        hasRefreshToken(userId: userId) && hasBioAccess
    }

    func disableBiometricAccessForRefreshToken(userId: String) {
        if availableUserIds.contains(userId) {
            hasBioAccess = false
        }
    }

    func clear(userId: String) {
        hasBioAccess = false
        availableUserIds.remove(userId)
        refreshTokensForUserIds.removeValue(forKey: userId)
    }

    func getStoredRefreshToken(userId: String, password: String, completion: @escaping (Result<JWT, TIMError>) -> Void) {
        if let storedRefreshToken = refreshTokensForUserIds[userId] {
            if password == storedRefreshToken.password {
                completion(.success(storedRefreshToken.refreshToken))
            } else {
                completion(.failure(TIMError.storage(.encryptedStorageFailed(.keyServiceFailed(.badPassword)))))
            }
        } else {
            completion(.failure(TIMError.storage(.encryptedStorageFailed(.secureStorageFailed(.failedToLoadData("No data available"))))))
        }
    }

    func getStoredRefreshTokenViaBiometric(userId: String, willBeginNetworkRequests: WillBeginNetworkRequestsCallback?, completion: @escaping (Result<BiometricRefreshToken, TIMError>) -> Void) {
        notImplementedForTestYet()
    }

    func storeRefreshToken(_ refreshToken: JWT, withExistingPassword password: String, completion: @escaping (Result<Void, TIMError>) -> Void) {
        availableUserIds.insert(refreshToken.userId) // Store should always succeed.
        refreshTokensForUserIds[refreshToken.userId] = StoredMockRefreshToken(password: password, refreshToken: refreshToken)
        completion(.success(()))
    }

    func storeRefreshToken(_ refreshToken: JWT, withNewPassword newPassword: String, completion: @escaping (Result<TIMESKeyCreationResult, TIMError>) -> Void) {
        storeRefreshToken(refreshToken, withExistingPassword: newPassword, completion: { _ in
            completion(.success(TIMESKeyCreationResult(keyId: "mock", longSecret: "mock")))
        })
    }

    func enableBiometricAccessForRefreshToken(password: String, userId: String, completion: @escaping (Result<Void, TIMError>) -> Void) {
        notImplementedForTestYet()
    }

    func enableBiometricAccessForRefreshToken(longSecret: String, userId: String) -> Result<Void, TIMError> {
        notImplementedForTestYet()
        return .failure(TIMError.storage(.encryptedStorageFailed(.secureStorageFailed(.authenticationFailedForData))))
    }

    func storeRefreshTokenWithLongSecret(_ refreshToken: JWT, longSecret: String, completion: @escaping (Result<Void, TIMError>) -> Void) {
        notImplementedForTestYet()
    }

    private func notImplementedForTestYet() {
        XCTFail("TestMock implementation is not implemented for this function yet!")
    }
}
