// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import SharedServices
import GMAppData
import ErrorData
import DependencyInjection

public class DataCardServiceMock: DataCardService {

    public var dataCards: [DataCard] = DataCardServiceMock.defaultStubs()

    public init() { }

    public func fetchDataCards() -> AnyPublisher<[DataCard], ServiceError> {
        MockPublisher.mockFailable(
            value: dataCards,
            error: ServiceError.networkError(.missingInternetConnection)
        )
    }

    static private var triforkOrganisation: HealthCareOrganization {
        HealthCareOrganization(name: "Trifork", sorId: "1234567890")
    }

    static public func defaultStubs() -> [DataCard] {
        [
            dataCard(
                status: .active,
                date: Date().addingTimeInterval(3600 * 24 * 30),
                episodeOfCareId: "1"
            ),
            dataCard(
                status: .closed,
                date: Date().addingTimeInterval(-3600 * 24 * 30 * 12 * 2),
                episodeOfCareId: "2"
            ),
            dataCard(
                status: .closed,
                date: Date().addingTimeInterval(-3600 * 24 * 30 * 12 * 5),
                episodeOfCareId: "3"
            )
        ]
    }

    static public func dataCard(status: DataCard.DocumentStatus, date: Date, episodeOfCareId: String) -> DataCard {
        DataCard(
            modifiedTime: date,
            author: triforkOrganisation,
            dirty: nil,
            episodeOfCareIdentifier: episodeOfCareId,
            documentUniqueId: nil,
            version: nil,
            documentStatus: status,
            closeDateTime: date,
            recordTarget: CprIdentifiedPerson(
                name: PersonName(
                    prefixName: nil,
                    firstNames: ["Test"],
                    middleNames: ["Borger"],
                    lastName: "Testsen"
                ),
                identifier: "1234567890"
            ),
            childPartnerSection: ChildPartnerSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                name: PersonName(prefixName: nil, firstNames: ["Partner"], middleNames: [], lastName: "Testsen"),
                identifier: "1234567891"
            ),
            languageSection: LanguageSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                nationality: "da",
                translationRequirements: TranslationRequirements(needsTranslation: ._false, language: "da"),
                additionalInformations: nil
            ),
            bodyMeasuresSection: BodyMeasuresSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                prenatalBodyWeight: IntegerValueWithUnitEnum(value: 3, unit: .kg),
                bodyHeight: IntegerValueWithUnitEnum(value: 56, unit: .cm),
                bodyMassIndex: DoubleValueWithUnitEnum(value: 25.0, unit: .kgM2)
            ),
            socialHistorySection: SocialHistorySection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                maritalOrPartnershipStatus: SocialHistorySection.MaritalOrPartnershipStatus.married,
                livingConditions: SocialHistorySection.LivingConditions.partnership
            ),
            previousBirthsSection: PreviousBirthsSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                births: [
                    PreviousBirthsSectionAllOfBirths(
                        children: [
                            PreviousBirthsSectionAllOfChildren(
                                sex: PreviousBirthsSectionAllOfChildren.Sex.female,
                                birthWeight: IntegerValueWithUnitEnum(value: 123, unit: .kg),
                                outcome: PreviousBirthsSectionAllOfChildren.Outcome.stillbirth,
                                childsCurrentHealthStatus: nil
                            )
                        ],
                        year: 2021,
                        gestationLength: IntegerValueWithUnitEnum(value: 123, unit: .d),
                        gestationLengthWeeks: IntegerValueWithUnitEnum(value: 30, unit: .w),
                        birthPlace: "Skejby",
                        pregnancySummary: nil,
                        birthSummary: nil,
                        subjectiveBirthExperience: PreviousBirthsSectionAllOfBirths.SubjectiveBirthExperience.normal,
                        puerperalSepsis: nil
                    )
                ]
            ),
            previousAbortionsSection: PreviousAbortionsSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                abortions: []
            ),
            expectedDateOfDeliverySection: ExpectedDateOfDeliverySection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                expectedDateOfDeliveryFromLastPeriod: ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod(
                    lastPeriod: date,
                    menstrualCycle: ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriodMenstrualCycle(
                        min: IntegerValueWithUnitEnum(value: 123, unit: .d),
                        max: IntegerValueWithUnitEnum(value: 150, unit: .d)
                    ),
                    expectedDateOfDelivery: date,
                    certainty: .certain
                ),
                offsetDate: nil,
                dateOfDelivery: date,
                gestationAge: "X+Y",
                trimester: 3,
                monthsIntoPregnancy: 7.34,
                week: 25,
                remarks: nil
            ),
            ultrasoundSection: UltrasoundSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: false,
                expectedDateOfDeliveryFromPrenatalUltrasoundScan: date
            ),
            prenatalRiskAssessmentSection: PrenatalRiskAssessmentSection(
                riskAssessment1: RiskAssessment1(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    familyHistory: nil,
                    consanguinity: nil,
                    firstTrimesterUltrasoundScanOfFetusRequested: true,
                    secondTrimesterUltrasoundScanOfFetusRequested: false,
                    prenatalRiskAssessmentRequested: true
                ),
                riskAssessment2: RiskAssessment2(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    multiplicityRiskAssessment: RiskAssessment2AllOfMultiplicityRiskAssessment(risk: ._true, comment: nil)
                ),
                riskAssessment3: RiskAssessment3(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    doubleTestCompleted: true,
                    doubleTestCompletedDate: date,
                    firstTrimesterUltrasoundScanOfFetusCompleted: date,
                    downsScreeningProbabilityAssessment: .high
                ),
                riskAssessment4: RiskAssessment4(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    secondTrimesterUltrasoundScanOfFetusCompleted: date
                ),
                riskAssessment5: RiskAssessment5(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    placentalBiopsiObservation: RiskAssessment5AllOfPlacentalBiopsiObservation(
                        date: date,
                        result: "Resultatet for en måling."
                    )
                ),
                riskAssessment6: RiskAssessment6(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    prenatalAmniocentesisiObservation: RiskAssessment5AllOfPlacentalBiopsiObservation(
                        date: date,
                        result: "Resultatet for en anden måling."
                    )
                )
            ),
            workEnvironmentImpactSection: WorkEnvironmentImpactSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                mother: WorkEnvironmentImpactSectionAllOfMother(
                    jobType: "Kontormus",
                    hoursOfWorkPerWeek: 37,
                    exposureToWorkEnvironmentRisk: WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk(
                        start: date,
                        end: nil,
                        calculated: IntegerValueWithUnitEnum(value: 123, unit: .g),
                        remarks: nil,
                        types: [
                            WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types.biological,
                            WorkEnvironmentImpactSectionAllOfMotherExposureToWorkEnvironmentRisk.Types.ergonomic
                        ]
                    ),
                    referredToOccupationalHealthDepartmentForRiskAssessment: false,
                    prenatalSickLeave: WorkEnvironmentImpactSectionAllOfMother.PrenatalSickLeave.full
                ),
                partner: WorkEnvironmentImpactSectionAllOfPartner(jobType: "CEO")
            ),
            allergiesSection: AllergiesSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                allergiesOfMother: nil,
                allergicPredispositionOfChild: nil
            ),
            screeningsSection: DataCardAllOfScreeningsSection(
                hepatitisB: posNegDeclinedResult(.negative, date: date),
                hiv: posNegDeclinedResult(.positive, date: date),
                syphilis: posNegDeclinedResult(.declined, date: date),
                chlamydia: posNegResult(.negative, date: date),
                gonorrhea: posNegResult(.positive, date: date),
                streptococcusGroupB: posNegResult(.negative, date: date),
                hemoglobinopathy: posNegResult(.positive, date: date),
                vitaminDConsumption: DateValueUnitScreeningSection(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: false,
                    date: date,
                    value: 1234,
                    unit: ""
                ),
                maternalRhD: MaternalRhDScreeningSection(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    rhesusType: .negative
                ),
                maternalAntibody: MaternalAntibodyScreeningSection(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    irregularBloodGroupAntibodyPresent: .positive
                ),
                maternalBloodtype: MaternalBloodtypeScreeningSection(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    bloodSampleTaken: ._false
                ),
                fetalRhDAntibody: FetalRhDAntibodyScreeningSection(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    rhesusType: .negative,
                    antiDIsoimmunizationAffectingPregnancy: .positive
                ),
                diabetesWeek18To20: DateValueUnitScreeningSection(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    date: date,
                    value: 1234,
                    unit: nil
                ),
                diabetesWeek28To30: DateValueUnitScreeningSection(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    date: date,
                    value: 4321,
                    unit: nil
                ),
                diabetesGlycosuria: DateValueUnitScreeningSection(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    date: date,
                    value: 1111,
                    unit: nil
                ),
                antiDImmunoglobulinGiven: YesNoDateScreeningSection(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    date: date,
                    value: ._true
                ),
                mrsaSwap: YesNoDateScreeningSection(
                    modifiedTime: date,
                    author: triforkOrganisation,
                    dirty: nil,
                    date: date,
                    value: ._false
                )
            ),
            chronicDiseasesSection: ChronicDiseasesSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                conditions: nil,
                comment: nil
            ),
            medicineSection: MedicineSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                realMedicine: "Da real ztuff",
                natureMedicine: nil
            ),
            mmrVaccinationSection: MmrVaccinationSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                status: .vaccinated
            ),
            historyOfPreviousTreatmentWithImpactOnPregnancySection: HistoryOfPreviousTreatmentWithImpactOnPregnancySection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                history: "historien ..."
            ),
            tobaccoUseAndExposureSection: TobaccoUseAndExposureSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                exposureToEnvironmentalTobaccoSmoke: ._false,
                tobaccoUser: .formerLongAgo,
                endDate: date.addingTimeInterval(-3600*24*30*60),
                smokeDuringPregnancy: ._false,
                referredToTobaccoCessationEducation: ._false
            ),
            alcoholDrinkingBehaviorSection: AlcoholDrinkingBehaviorSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                weeklyConsumptionBeforePregnancy: WeeklyAlcoholUnitsConsumption(
                    consumption: IntegerValueWithUnitEnum(value: 123, unit: .d),
                    consumesAlcohol: ._false
                ),
                weeklyConsumptionDuringPregnancy: WeeklyConsumptionDuringPregnancy(
                    consumption: IntegerValueWithUnitEnum(value: 1, unit: .hWk),
                    consumesAlcohol: ._false,
                    excessiveDrinkingCount: nil,
                    excessiveDrinkingWeeks: nil,
                    tweakPoints: nil
                )
            ),
            assistedFertilizationSection: AssistedFertilizationSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                remarks: nil,
                type: .eggDonation
            ),
            substanceUseSection: SubstanceUseSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                substanceUseBeforePregnancy: SubstanceUseObservation(
                    substanceUse: ._false,
                    text: nil
                ),
                substanceUseDuringPregnancy: SubstanceUseObservation(
                    substanceUse: ._true,
                    text: "Beskrivelse af noget vigtigt."
                )
            ),
            dietaryAndExerciseSection: DietaryAndExerciseSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: false,
                text: "Teksten om det hele..."
            ),
            healthInsuranceSection: HealthInsuranceSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                insuranceGroup: ._1,
                generalPractitioner: Person(name: PersonName(prefixName: nil, firstNames: ["Læge"], middleNames: [], lastName: "Vigtigsen")),
                generalPractitionerAddress: Address(streetAndNumber: "Testvej", city: "Testby", zip: "8888"),
                generalPractitionerTelephoneNumber: nil
            ),
            desiredBirthPlaceSection: DesiredBirthPlaceSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                desiredBirthPlace: "Skejby",
                homeBirth: false
            ),
            primaryPlannedBirthPlaceSection: PrimaryPlannedBirthPlaceSection(
                modifiedTime: Date(),
                author: triforkOrganisation,
                dirty: nil,
                primaryPlannedBirthPlace: triforkOrganisation
            ),
            revisedPlannedBirthPlaceSection: RevisedPlannedBirthPlaceSection(
                modifiedTime: Date(),
                author: triforkOrganisation,
                dirty: nil,
                revisedPlannedBirthPlace: triforkOrganisation
            ),
            midwifeClinicSection: MidwifeClinicSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                assignedMidwifeClinic: triforkOrganisation,
                parentalCraftClassDesired: nil
            ),
             midwifeConsultationSection: MidwifeConsultationSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                midwifeConsultationWishes: MidwifeConsultationSectionAllOfMidwifeConsultationWishes(
                    location: "Somewhere",
                    midwife: "Jordemoder Testsen",
                    weekday: .wednesday
                )
             ),
            overallAssessmentSection: OverallAssessmentSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                prenatalCareAssessment: OverallAssessmentSectionAllOfPrenatalCareAssessment(
                    assessment: "1234",
                    comment: "Kommentar omkring det.",
                    careLevel: .basic
                ),
                obstetricAssessment: ObstetricAssessment(_required: ._true, reason: nil),
                socialServicesPerinatalCareAssessment: SocialServicesPerinatalCareAssessment(
                    _required: ._false,
                    reason: nil,
                    assessmentDate: nil
                )
            ),
            multiplePregnancySection: MultiplePregnancySection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                numberOfFetuses: Int.random(in: 1...4)
            ),
            personalDataSection: PersonalDataSection(
                name: PersonName(prefixName: nil, firstNames: ["Gravid"], middleNames: ["Mave"], lastName: "Testsen"),
                cpr: "1234567890",
                address: nil,
                temporaryAddress: nil,
                telecoms: nil,
                generalPractitioner: PersonalDataSectionGeneralPractitioner(
                    name: "Praktiserende læge",
                    address: Address(
                        streetAndNumber: "Europaplads",
                        city: "Aarhus",
                        zip: "8000"
                    ),
                    telephoneNumber: "88888888"),
                nextOfKin: nil
            ),
            healthcareSection: HealthCareSection(
                modifiedTime: date,
                author: triforkOrganisation,
                dirty: nil,
                ward: triforkOrganisation,
                healthNurse: HealthCareNurse(phone: "12345678", name: "Call me maybe"
                )
            )
        )
    }
    static private func posNegDeclinedResult(_ result: PositiveNegativeDeclinedScreeningWithDateSection.Result, date: Date) -> PositiveNegativeDeclinedScreeningWithDateSection {
        PositiveNegativeDeclinedScreeningWithDateSection(
            modifiedTime: date,
            author: triforkOrganisation,
            dirty: nil,
            date: date,
            result: result
        )
    }

    static private func posNegResult(_ result: PosNegEnum, date: Date) -> PositiveNegativeScreeningWithDateSection {
        PositiveNegativeScreeningWithDateSection(
            modifiedTime: date,
            author: triforkOrganisation,
            dirty: nil,
            date: date,
            result: result
        )
    }
}
